//	FileLoader.cpp - Derivative of FileBasedDocument, used to load and save
//					 collections of Heilan command line parameters.
//	----------------------------------------------------------------------------
//	Copyright (c) 2006-2007 Niall Moody.
//
//	This program is free software; you can redistribute it and/or modify
//	it under the terms of the GNU General Public License as published by
//	the Free Software Foundation; either version 2 of the License, or
//	(at your option) any later version.
//
//	This program is distributed in the hope that it will be useful,
//	but WITHOUT ANY WARRANTY; without even the implied warranty of
//	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//	GNU General Public License for more details.
//
//	You should have received a copy of the GNU General Public License
//	along with this program; if not, write to the Free Software
//	Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//	----------------------------------------------------------------------------

#include "juce_amalgamated.h"
#include "FileLoader.h"
#include "window.h"

File FileLoader::lastDocument;

//------------------------------------------------------------------------------
FileLoader::FileLoader(HeilanFrontEnd *parentComp):
FileBasedDocument(T(".hfe"),
				  T("*.hfe"),
				  T("Choose a file to open..."),
				  T("Choose a file to save to...")),
parent(parentComp)
{
	String tempstr;

	tempstr = File::getSpecialLocation(File::userApplicationDataDirectory).getFullPathName();
	tempstr << File::separator << T("HeilanFrontEndDefaults.hfe");

	{
		File phil(tempstr);

		if(!phil.exists())
			saveAs(phil, false, false, false);

		loadFrom(phil, true);
	}
}

//------------------------------------------------------------------------------
FileLoader::~FileLoader()
{
	
}

//------------------------------------------------------------------------------
const String FileLoader::getDocumentTitle()
{
	return T("Heilan Front End Document");
}

//------------------------------------------------------------------------------
const String FileLoader::loadDocument(const File& file)
{
	String retval;
	XmlDocument doc(file);
	XmlElement *tempEl = doc.getDocumentElement();
	XmlElement *tempEl2;

	retval << T("Could not load file: ") << file.getFullPathName();

	if(tempEl)
	{
		if(tempEl->hasTagName(T("HeilanFrontEnd")))
		{
			//Graphics options.
			tempEl2 = tempEl->getChildByName(T("windowWidth"));
			if(tempEl2)
				parent->setWindowWidth(tempEl2->getStringAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("windowHeight"));
			if(tempEl2)
				parent->setWindowHeight(tempEl2->getStringAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("antialias"));
			if(tempEl2)
				parent->setAntialias(tempEl2->getIntAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("fullscreen"));
			if(tempEl2)
				parent->setFullscreen(tempEl2->getBoolAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("bloomFilter"));
			if(tempEl2)
				parent->setBloomFilter(tempEl2->getBoolAttribute(T("val")));

			//Audio options.
			tempEl2 = tempEl->getChildByName(T("audioApi"));
			if(tempEl2)
			{
				parent->setAudioApi(tempEl2->getIntAttribute(T("val")));
				parent->updateDevices();
				parent->repaint();
			}
			tempEl2 = tempEl->getChildByName(T("audioDevice"));
			if(tempEl2)
				parent->setAudioDevice(tempEl2->getIntAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("audioInputDevice"));
			if(tempEl2)
				parent->setAudioInputDevice(tempEl2->getIntAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("audioBlockSize"));
			if(tempEl2)
				parent->setAudioBlockSize(tempEl2->getIntAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("audioSamplerate"));
			if(tempEl2)
				parent->setAudioSamplerate(tempEl2->getIntAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("audioSpeakers"));
			if(tempEl2)
				parent->setAudioSpeakers(tempEl2->getIntAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("audioSpeakersFile"));
			if(tempEl2)
				parent->setAudioSpeakersFile(tempEl2->getStringAttribute(T("val")));

			//Other options.
			tempEl2 = tempEl->getChildByName(T("oscPort"));
			if(tempEl2)
				parent->setOscPort(tempEl2->getStringAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("oscInputAddress"));
			if(tempEl2)
				parent->setOscInputAddress(tempEl2->getStringAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("oscOutputPort"));
			if(tempEl2)
				parent->setOscOutputPort(tempEl2->getStringAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("oscOutputAddress"));
			if(tempEl2)
				parent->setOscOutputAddress(tempEl2->getStringAttribute(T("val")));

			tempEl2 = tempEl->getChildByName(T("disableSound"));
			if(tempEl2)
				parent->setDisableSound(tempEl2->getBoolAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("disableOsc"));
			if(tempEl2)
				parent->setDisableOsc(tempEl2->getBoolAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("heilanPath"));
			if(tempEl2)
				parent->setHeilanPath(tempEl2->getStringAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("recordOscPath"));
			if(tempEl2)
				parent->setRecordOscPath(tempEl2->getStringAttribute(T("val")));
			tempEl2 = tempEl->getChildByName(T("enableCursor"));
			if(tempEl2)
				parent->setEnableCursor(tempEl2->getBoolAttribute(T("val")));

			//Libraries.
			tempEl2 = tempEl->getChildByName(T("library"));
			while(tempEl2)
			{
				parent->addLibraryText(tempEl2->getStringAttribute(T("val")));
				tempEl2 = tempEl2->getNextElement();
				while(tempEl2 != 0)
				{
					if(tempEl2->getTagName() != T("library"))
						tempEl2 = tempEl2->getNextElement();
				}
			}

			//File to load.
			tempEl2 = tempEl->getChildByName(T("file"));
			if(tempEl2)
				parent->setFile(tempEl2->getStringAttribute(T("val")));

			retval = T("");
		}

		delete tempEl;
	}

	return retval;
}

//------------------------------------------------------------------------------
const String FileLoader::saveDocument(const File& file)
{
	String retval;
	XmlElement rootEl(T("HeilanFrontEnd"));
	XmlElement *tempEl;

	retval << T("Could not save to file: ") << file.getFullPathName();

	//Graphics options.
	tempEl = new XmlElement(T("windowWidth"));
	tempEl->setAttribute(T("val"), parent->getWindowWidth());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("windowHeight"));
	tempEl->setAttribute(T("val"), parent->getWindowHeight());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("antialias"));
	tempEl->setAttribute(T("val"), parent->getAntialias());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("fullscreen"));
	tempEl->setAttribute(T("val"), parent->getFullscreen());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("bloomFilter"));
	tempEl->setAttribute(T("val"), parent->getBloomFilter());
	rootEl.addChildElement(tempEl);

	//Audio options.
	tempEl = new XmlElement(T("audioApi"));
	tempEl->setAttribute(T("val"), parent->getAudioApi());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("audioDevice"));
	tempEl->setAttribute(T("val"), parent->getAudioDevice());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("audioInputDevice"));
	tempEl->setAttribute(T("val"), parent->getAudioInputDevice());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("audioBlockSize"));
	tempEl->setAttribute(T("val"), parent->getAudioBlockSize());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("audioSamplerate"));
	tempEl->setAttribute(T("val"), parent->getAudioSamplerate());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("audioSpeakers"));
	tempEl->setAttribute(T("val"), parent->getAudioSpeakers());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("audioSpeakersFile"));
	tempEl->setAttribute(T("val"), parent->getAudioSpeakersFile());
	rootEl.addChildElement(tempEl);

	//Other options.
	tempEl = new XmlElement(T("oscPort"));
	tempEl->setAttribute(T("val"), parent->getOscPort());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("oscInputAddress"));
	tempEl->setAttribute(T("val"), parent->getOscInputAddress());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("oscOutputPort"));
	tempEl->setAttribute(T("val"), parent->getOscOutputPort());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("oscOutputAddress"));
	tempEl->setAttribute(T("val"), parent->getOscOutputAddress());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("disableSound"));
	tempEl->setAttribute(T("val"), parent->getDisableSound());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("disableOsc"));
	tempEl->setAttribute(T("val"), parent->getDisableOsc());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("heilanPath"));
	tempEl->setAttribute(T("val"), parent->getHeilanPath());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("recordOscPath"));
	tempEl->setAttribute(T("val"), parent->getRecordOscPath());
	rootEl.addChildElement(tempEl);
	tempEl = new XmlElement(T("enableCursor"));
	tempEl->setAttribute(T("val"), parent->getEnableCursor());
	rootEl.addChildElement(tempEl);

	//Libraries.
	for(int i=0;i<parent->getNumLibraries();++i)
	{
		tempEl = new XmlElement(T("library"));
		tempEl->setAttribute(T("val"), parent->getLibrary(i));
		rootEl.addChildElement(tempEl);
	}

	//File to load.
	tempEl = new XmlElement(T("file"));
	tempEl->setAttribute(T("val"), parent->getFile());
	rootEl.addChildElement(tempEl);

	if(rootEl.writeToFile(file, T("")))
		retval = T("");

	return retval;
}

//------------------------------------------------------------------------------
const File FileLoader::getLastDocumentOpened()
{
	return lastDocument;
}

//------------------------------------------------------------------------------
void FileLoader::setLastDocumentOpened(const File& file)
{
	lastDocument = file;
}
